% This code computes the MA representation and impulse responses to 
% increases in the innovations of a reduced form VAR (without trend)

% Input
% VAR: VAR estimates form VAREst
% Maxhor: Maximum number of horizons for impulse responses
% Impact: Impact effect of a reduced-form shock (0 for impact increase of
% 1, or 1 for a one std increase on impact
%
% Output
% Out.VARnobs 
% Out.VARlag 
% Out.VARnvars 
% Out.VARcoef 
% Out.Constant 
% Out.uncondmean
% Out.VARVCV 
% Out.MAcoef 
% Out.Companion
% Out.Horizon = Maxhor
% Out.IRFs   --- Impulse Response Functions
% Out.Data = VAR.data

% Note: IRFs are response of an increase of the innovation by 1 (expressed
% as percent deviation vis-a-vis the unconditional mean)

% (C) Laurent Kemoe, October 2014


function Out = VARIrf(VAR, Maxhor, Impact)

p = VAR.lag;
N = VAR.nvars;
T = VAR.nobs;
EST = VAR.Coef;
Const = VAR.Constant;
Omega = VAR.VCV;

if Const == 1
    C = VAR.Const;
    fst = 2;
else
    fst = 1;
end

% Companion form of the VAR
F = VAR.Companion;


PSIF = zeros(size(F,1),size(F,2), Maxhor+fst); 
PSI = zeros(N,N,Maxhor+fst); % Index 1 -> variable; Index 2 -> shock; Index 3 -> horizon
if Const == 1
    PSIF(:,:,1) = eye(size(F))/(eye(size(F))-F); % matrix that multiplies the constant (if any) in the MA representation
    PSIF(:,:,2) = eye(size(F));
else
    PSIF(:,:,1) = eye(size(F));
end

for i = 1:Maxhor
    PSIF(:,:,fst+i) = F*PSIF(:,:,fst+i-1);
end

for i = 1:Maxhor+fst
    PSI(:,:,i) = PSIF(1:N,1:N,i);
end

Out.VARnobs = T;
Out.VARlag = p;
Out.VARnvars = N;
Out.VARcoef = EST;
Out.Constant = Const;
Out.VARVCV = Omega;
Out.Horizon = Maxhor;
Out.Companion = real(F);
Out.Data = VAR.data;
Out.Impact = Impact;

if Const == 1
    Out.uncondmean = PSI(:,:,1)*C;
else
    Out.uncondmean = zeros(N,1);
end

if Const == 1
    PSI = PSI(:,:,2:end);
end

Out.MAcoef = real(PSI); % MA representation of the reduced-form VAR

A0 = chol(Omega, 'lower');
% for i = 1:Maxhor+1
%     PSI(:,:,i) = A0*PSI(:,:,i);
% end
% Out.MAcoef = PSI;
if nargin == 3
    if Impact == 0
        shock = eye(N);
    elseif Impact == 1
        shock = diag((diag(A0)));
    else
        error('The impact effect is either 0 for unit increase of innovations or 1 for one std increase')
    end
    for h = 1:Maxhor+1
        for j = 1:N
            PSI(:,j,h) = PSI(:,:,h)*shock(:,j);
        end
    end
end
Out.IRFs = real(PSI); % Reduced-form IRF in percent


