% This function estimates a VAR(p) using OLS
% Input: 
% Y: data 
% p: lag length
% Optional:  Const (1: default -> include a constant; 0 -> don't), 
% Note: Any trend in the data should be removed before using this function

% Output
% Out.data --- Raw data
% Out.lag --- Number of lags
% Out.nobs --- Number of observations
% Out.nvars --- Number of variables
% Out.Coef --- Estimated coefficients
% Out.Residuals --- Residuals         
% Out.VCV --- Variance covariance matrix of residuals
% Out.Companion --- Companion matrix
% Out.numbeta --- Number of parameters estimated
% Out.reg --- Regressors without constant if any
% Out.regCT --- Regressors with constant if any

% (C) Laurent Kemoe, October 2014

function Out = VAREst(Y, p, const)%, trend,meth)

Out.data = Y;
Out.lag = p;
[T, N] = size(Y);
Out.nobs = T;
Out.nvars = N;

TT = T-p;

if (nargin > 2)
    if isempty(const)
         Out.Constant = 1;
    else
        Out.Constant = const;
    end
else
    Out.Constant = 1;
end
if Out.Constant == 0
    cst = [];
else
    cst = ones(TT,1);
end

YY = Y(p+1:end,:);
XX = [];
for i = 1:p
    XX = [Y(i:TT+i-1,:) XX];
end
Out.reg = XX; % Regressors and w/o constant

XX = [cst XX];
Out.regCT = XX; % Regressors with constant if any

Resid = zeros(size(YY));
nb = size(XX,2);

% OLS Estimation
OLSC = zeros(size(XX,2),N);

for i = 1:N
    OLSC(:,i) = (XX'*XX)\(XX'*YY(:,i));
    Resid(:,i) = YY(:,i) - XX*OLSC(:,i);
end
Out.Coef = OLSC';
EST = Out.Coef;
Out.VCV = Resid'*Resid/(TT-nb);

Out.Residuals = Resid;

if const == 1
    Out.Const = EST(:,1);
    fst = 2;
else
    fst = 1;
    Out.Const = [];
end

% Companion form of the VAR
Out.Companion = [EST(:,fst:end); eye(N*(p-1),N*p)];
Out.numbeta = nb;

